/*  
*/

#define VERSION "1.00" 

// Relays according to the schamatics:
#define pin_rel_1 A0  // Rel_1
#define pin_rel_2 A1  // Rel_2
#define pin_rel_3 A2  // Rel_3
#define pin_rel_4 A3  // Rel_4
#define pin_rel_5 A4  // Rel_5

// Connected to Spider Keyer:
#define pin_ptt_in 2  // Connected to transceiver via CINCH on the rear

// Small box w/ buttons:
#define pin_button_green 3  // 
#define pin_button_a 4  //
#define pin_button_b 5  //


#define pin_ptt_out_tcvr 13  // CINCH on the rear panel
#define pin_ptt_out_a 11 // Cable coming to PA A (main)
#define pin_ptt_out_b 12 // Cable coming to PA A (sub)

#define pin_led_multibeaming 6 // A+B
#define pin_led_singlebeaming 7 // A/B
#define pin_led_ant_a 8 // Ant A selected
#define pin_led_ant_b 9 // Ant A selected
#define pin_led_ptt 10 // PTT

const unsigned long DEBOUNCE_TIME = 20;
const long TOGGLE_TIME = 600;
const byte BUTTON_GREEN = 0;
const byte BUTTON_A = 1;
const byte BUTTON_B = 2;

bool b1, b2, b3; 
bool btn_1, btn_2, btn_3, prev_btn_1, prev_btn_2, prev_btn_3, prev_b1, prev_b2, prev_b3;
int button_selected = BUTTON_A;   // Start switched to A output
unsigned long debounce_b1, debounce_b2, debounce_b3;
unsigned long btn_1_down, btn_2_down, btn_3_down;
bool multibeaming_on = false;
bool ptt_tcvr_on = false;
bool output_a_on = false; // Current final status 
bool output_b_on = false; // Current final status 
bool locked = false; // No changes when PTT is ON (transmitting)
bool testing_mode = false;

// --------------------------------------------------------------------------------------------
void setup() {
  int i;
  
  // Initialize pins:
  pinMode(pin_rel_1, OUTPUT);
  pinMode(pin_rel_2, OUTPUT);
  pinMode(pin_rel_3, OUTPUT);
  pinMode(pin_rel_4, OUTPUT);
  pinMode(pin_rel_5, OUTPUT);
 
  pinMode(pin_ptt_in, INPUT_PULLUP);

  pinMode(pin_button_green, INPUT_PULLUP);
  pinMode(pin_button_a, INPUT_PULLUP);
  pinMode(pin_button_b, INPUT_PULLUP);

  pinMode(pin_ptt_out_tcvr, OUTPUT);
  pinMode(pin_ptt_out_a, OUTPUT);
  pinMode(pin_ptt_out_b, OUTPUT);

  pinMode(pin_led_multibeaming, OUTPUT);
  pinMode(pin_led_singlebeaming, OUTPUT);
  pinMode(pin_led_ant_a, OUTPUT);
  pinMode(pin_led_ant_b, OUTPUT);
  pinMode(pin_led_ptt, OUTPUT);

  /*
  for (i = 0; i < 9; i++) {
    digitalWrite(pin_rel_1, true); delay(10);
    digitalWrite(pin_rel_2, true); delay(10);
    digitalWrite(pin_rel_3, true); delay(10);
    digitalWrite(pin_rel_4, true); delay(10);
    digitalWrite(pin_rel_5, true); delay(10);
    digitalWrite(pin_rel_1, false); delay(10);
    digitalWrite(pin_rel_2, false); delay(10);
    digitalWrite(pin_rel_3, false); delay(10);
    digitalWrite(pin_rel_4, false); delay(10);
    digitalWrite(pin_rel_5, false); delay(10);
  }
*/

  b1 = false;
  b2 = false;
  b3 = false;

  prev_b1 = false;
  prev_b2 = false;
  prev_b3 = false;

  btn_1 = false;
  btn_2 = false;
  btn_3 = false;

  prev_btn_1 = false;
  prev_btn_2 = false;
  prev_btn_3 = false;
  
  btn_1_down = 0;
  btn_2_down = 0;
  btn_3_down = 0; 

  debounce_b1 = 0; // 4294967294; // Maximum possible value 
  debounce_b2 = 0;
  debounce_b3 = 0;


  digitalWrite(pin_led_multibeaming, false);
  digitalWrite(pin_led_singlebeaming, false);
  digitalWrite(pin_led_ant_a, false);
  digitalWrite(pin_led_ant_b, false);
  digitalWrite(pin_led_ptt, false);


  
/*  if (testing_mode) 
    for (i = 0; i < 3; i++) { */
  // The goal is to make some delay for the digitalRead(pin_button_green) to work reliably:
  digitalWrite(pin_led_multibeaming, true);
  digitalWrite(pin_rel_1, true); delay(30);
  digitalWrite(pin_rel_1, false); delay(30);
  digitalWrite(pin_led_multibeaming, false);
  
  digitalWrite(pin_led_singlebeaming, true);
  digitalWrite(pin_rel_2, true); delay(30);
  digitalWrite(pin_rel_2, false); delay(30);
  digitalWrite(pin_led_singlebeaming, false);
  
  digitalWrite(pin_led_ant_a, true);
  digitalWrite(pin_rel_3, true); delay(30);
  digitalWrite(pin_rel_3, false); delay(30);
  digitalWrite(pin_led_ant_a, false);
  
  digitalWrite(pin_led_ant_b, true);
  digitalWrite(pin_rel_4, true); delay(30);
  digitalWrite(pin_rel_4, false); delay(30);
  digitalWrite(pin_led_ant_b, false);
  
  digitalWrite(pin_led_ptt, true);
  digitalWrite(pin_rel_5, true); delay(30);
  digitalWrite(pin_rel_5, false); delay(30); 
  digitalWrite(pin_led_ptt, false);
  

  // Keeping green button pressed during power-on switches to the testing mode:
  testing_mode = ! digitalRead(pin_button_green); 

  
}

// --------------------------------------------------------------------------------------------
void loop() {


  ptt_tcvr_on = ! digitalRead(pin_ptt_in);



  // Do buttons debounce:
  b1 = ! digitalRead(pin_button_green);
  if (b1 != prev_b1) debounce_b1 = millis();
  if ((millis() - debounce_b1) > DEBOUNCE_TIME) { 
    if (b1) button_selected = BUTTON_GREEN;
    btn_1 = b1;
  }
  b2 = ! digitalRead(pin_button_a);
  if (b2 != prev_b2) debounce_b2 = millis();
  if ((millis() - debounce_b2) > DEBOUNCE_TIME) {
    if (b2) button_selected = BUTTON_A;
    btn_2 = b2;
  }
  b3 = ! digitalRead(pin_button_b);
  if (b3 != prev_b3) debounce_b3 = millis();
  if ((millis() - debounce_b3) > DEBOUNCE_TIME) {
    if (b3) button_selected = BUTTON_B;
    btn_3 = b3;
  }

  if (! testing_mode) {
    // Save times when buttons were depressed:
    // Button 1 (green) was just depressed:         
    if (! prev_btn_1 && btn_1) { btn_1_down = millis(); }
    // Button 2 was just depressed:         
    if (! prev_btn_2 && btn_2) { btn_2_down = millis(); }
    // Button 3 was just depressed:         
    if (! prev_btn_3 && btn_3) { btn_3_down = millis(); }
  
    // Toggle multibeaming by long press of buttons:
    // Button 1 (green) was just released:         
    if (btn_1 && (millis() > btn_1_down + TOGGLE_TIME)) { // after some time:
        multibeaming_on = true;
    }
    // Button 2 was just released:         
    if (btn_2 && (millis() > btn_2_down + TOGGLE_TIME)) { // after some time:
        multibeaming_on = false;
    }
    // Button 3 was just released:         
    if (btn_3 && (millis() > btn_3_down + TOGGLE_TIME)) { // after some time:
        multibeaming_on = false;
      }
  
  
    // Set PTTs for both outputs:
    if (! ptt_tcvr_on) { // currently RX mode
      output_a_on = button_selected == BUTTON_GREEN || button_selected == BUTTON_A;
      output_b_on = button_selected == BUTTON_GREEN || button_selected == BUTTON_B;
    } else { // currently TX mode
      output_a_on = multibeaming_on || button_selected == BUTTON_A || button_selected == BUTTON_GREEN;
      output_b_on = multibeaming_on || button_selected == BUTTON_B || button_selected == BUTTON_GREEN;
    }
  
    digitalWrite(pin_led_multibeaming, multibeaming_on);
    digitalWrite(pin_led_singlebeaming, ! multibeaming_on);
  
    digitalWrite(pin_led_ant_a, output_a_on);
    digitalWrite(pin_led_ant_b, output_b_on);
  
    
    digitalWrite(pin_led_ptt, ptt_tcvr_on);
  
    digitalWrite(pin_ptt_out_tcvr, ptt_tcvr_on); 
  
  
  
  
    if (! locked) { // Don't change any output while TX is underway:
      
      digitalWrite(pin_ptt_out_a, ptt_tcvr_on && output_a_on); 
      digitalWrite(pin_ptt_out_b, ptt_tcvr_on && output_b_on); 
    
      if (! ptt_tcvr_on) { // RX
        if (output_a_on && output_b_on) { // Mix both for RX
          digitalWrite(pin_rel_1, true); // 
          digitalWrite(pin_rel_2, true); // 
          digitalWrite(pin_rel_3, false); // 
          digitalWrite(pin_rel_4, false); // 
          digitalWrite(pin_rel_5, false); // 
          
        } else if (output_a_on) { // Listen to A only
          digitalWrite(pin_rel_1, false); // 
          digitalWrite(pin_rel_2, false); // 
          digitalWrite(pin_rel_3, false); // 
          digitalWrite(pin_rel_4, false); // Can be any
          digitalWrite(pin_rel_5, false); // Can be any
          
        } else { // output_b_on  // Listen to B only
          digitalWrite(pin_rel_1, false); // 
          digitalWrite(pin_rel_2, true); // 
          digitalWrite(pin_rel_3, true); // 
          digitalWrite(pin_rel_4, false); // Can be any
          digitalWrite(pin_rel_5, false); // Can be any
        }
      } else { // TX
        if (output_a_on && output_b_on) { // Mix both for TX
          digitalWrite(pin_rel_1, true); // 
          digitalWrite(pin_rel_2, true); // 
          digitalWrite(pin_rel_3, false); // 
          digitalWrite(pin_rel_4, false); // 
          digitalWrite(pin_rel_5, false); // 
          
        } else if (output_a_on) { // Transmit to A only
          digitalWrite(pin_rel_1, true); // 
          digitalWrite(pin_rel_2, true); // 
          digitalWrite(pin_rel_3, false); // Can be any
          digitalWrite(pin_rel_4, false); // 
          digitalWrite(pin_rel_5, true); // 
          
        } else { // output_b_on  // Transmit to B only
          digitalWrite(pin_rel_1, true); // 
          digitalWrite(pin_rel_2, true); // Can be any
          digitalWrite(pin_rel_3, false); // 
          digitalWrite(pin_rel_4, true); // 
          digitalWrite(pin_rel_5, false); // 
        }
        
      }
    }
    
  
  } else { // Testing mode
    // Copy the status of PTT to test connection TCVR-SWITCH:
    digitalWrite(pin_led_ptt, ptt_tcvr_on);

    digitalWrite(pin_ptt_out_a, btn_2); // Toggle PTT-A by button to test connection SWITCH-PA-A
    digitalWrite(pin_ptt_out_b, btn_3); // Toggle PTT-B by button to test connection SWITCH-PA-B
    // Switch LEDs accordingly:
    digitalWrite(pin_led_ant_a, btn_2); // 
    digitalWrite(pin_led_ant_b, btn_3); // 

    // Toggle all relays according the Green button:    
    digitalWrite(pin_rel_1, btn_1);
    digitalWrite(pin_rel_2, btn_1);
    digitalWrite(pin_rel_3, btn_1);
    digitalWrite(pin_rel_4, btn_1);
    digitalWrite(pin_rel_5, btn_1);

    // Toggle "beaming" LEDs to indicate the testing mode:
    digitalWrite(pin_led_multibeaming, (millis() / 250) % 2 == 0);
    digitalWrite(pin_led_singlebeaming, ! digitalRead(pin_led_multibeaming));
  }

  locked = ptt_tcvr_on;
  prev_b1 = b1;
  prev_b2 = b2;
  prev_b3 = b3;
  prev_btn_1 = btn_1;
  prev_btn_2 = btn_2;
  prev_btn_3 = btn_3;


}
// --------------------------------------------------------------------------------------------
